"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.precompile02 = void 0;
const util_1 = require("@ethereumjs/util");
const sha256_1 = require("ethereum-cryptography/sha256");
const evm_1 = require("../evm");
function precompile02(opts) {
    const data = opts.data;
    let gasUsed = opts._common.param('gasPrices', 'sha256');
    gasUsed += opts._common.param('gasPrices', 'sha256Word') * BigInt(Math.ceil(data.length / 32));
    if (opts._debug) {
        opts._debug(`Run KECCAK256 (0x02) precompile data=${(0, util_1.short)(opts.data)} length=${opts.data.length} gasLimit=${opts.gasLimit} gasUsed=${gasUsed}`);
    }
    if (opts.gasLimit < gasUsed) {
        if (opts._debug) {
            opts._debug(`KECCAK256 (0x02) failed: OOG`);
        }
        return (0, evm_1.OOGResult)(opts.gasLimit);
    }
    const hash = (0, util_1.toBuffer)((0, sha256_1.sha256)(data));
    if (opts._debug) {
        opts._debug(`KECCAK256 (0x02) return hash=${hash.toString('hex')}`);
    }
    return {
        executionGasUsed: gasUsed,
        returnValue: hash,
    };
}
exports.precompile02 = precompile02;
//# sourceMappingURL=02-sha256.js.map