"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.precompile06 = void 0;
const util_1 = require("@ethereumjs/util");
const evm_1 = require("../evm");
const bn128 = require('rustbn.js');
function precompile06(opts) {
    const inputData = opts.data.slice(0, 128);
    const gasUsed = opts._common.param('gasPrices', 'ecAdd');
    if (opts._debug) {
        opts._debug(`Run ECADD (0x06) precompile data=${(0, util_1.short)(opts.data)} length=${opts.data.length} gasLimit=${opts.gasLimit} gasUsed=${gasUsed}`);
    }
    if (opts.gasLimit < gasUsed) {
        if (opts._debug) {
            opts._debug(`ECADD (0x06) failed: OOG`);
        }
        return (0, evm_1.OOGResult)(opts.gasLimit);
    }
    const returnData = bn128.add(inputData);
    // check ecadd success or failure by comparing the output length
    if (returnData.length !== 64) {
        if (opts._debug) {
            opts._debug(`ECADD (0x06) failed: OOG`);
        }
        return (0, evm_1.OOGResult)(opts.gasLimit);
    }
    if (opts._debug) {
        opts._debug(`ECADD (0x06) return value=${returnData.toString('hex')}`);
    }
    return {
        executionGasUsed: gasUsed,
        returnValue: returnData,
    };
}
exports.precompile06 = precompile06;
//# sourceMappingURL=06-ecadd.js.map