"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.precompile08 = void 0;
const util_1 = require("@ethereumjs/util");
const evm_1 = require("../evm");
const bn128 = require('rustbn.js');
function precompile08(opts) {
    const inputData = opts.data;
    // no need to care about non-divisible-by-192, because bn128.pairing will properly fail in that case
    const inputDataSize = BigInt(Math.floor(inputData.length / 192));
    const gasUsed = opts._common.param('gasPrices', 'ecPairing') +
        inputDataSize * opts._common.param('gasPrices', 'ecPairingWord');
    if (opts._debug) {
        opts._debug(`Run ECPAIRING (0x08) precompile data=${(0, util_1.short)(opts.data)} length=${opts.data.length} gasLimit=${opts.gasLimit} gasUsed=${gasUsed}`);
    }
    if (opts.gasLimit < gasUsed) {
        if (opts._debug) {
            opts._debug(`ECPAIRING (0x08) failed: OOG`);
        }
        return (0, evm_1.OOGResult)(opts.gasLimit);
    }
    const returnData = bn128.pairing(inputData);
    // check ecpairing success or failure by comparing the output length
    if (returnData.length !== 32) {
        if (opts._debug) {
            opts._debug(`ECPAIRING (0x08) failed: OOG`);
        }
        // TODO: should this really return OOG?
        return (0, evm_1.OOGResult)(opts.gasLimit);
    }
    if (opts._debug) {
        opts._debug(`ECPAIRING (0x08) return value=${returnData.toString('hex')}`);
    }
    return {
        executionGasUsed: gasUsed,
        returnValue: returnData,
    };
}
exports.precompile08 = precompile08;
//# sourceMappingURL=08-ecpairing.js.map